import json
from typing import TYPE_CHECKING

from .keys import CATEGORIES
from .label import Label
from .logger import get_debug_logger

if TYPE_CHECKING:
    from . import DatalakeClient

datalake_logger = get_debug_logger('Annotation')


class Annotation:
    def __init__(self, client: "DatalakeClient"):
        self._client = client

    def upload_coco(self, file_path: str):
        # load json file
        f = open(file_path)
        coco_data = json.load(f)
        f.close()

        label = Label(client=self._client)
        label.create_label_from_cocojson(categories=coco_data[CATEGORIES])
        # TODO: format Images and Annotations as MetaUpdates format, then call metaUpdates API




import base64

from .annotation import Annotation
from .constants import BOX_ANNOTATION, POLYGON_ANNOTATION, LINE_ANNOTATION
from .datalakeinterface import DatalakeInterface
from .logger import get_debug_logger
from .model import Model

datalake_logger = get_debug_logger('DatalakeClient')


class DatalakeClient:
    """
    Python SDK of Datalake
    """

    def __init__(self, api_key: str, secret: str, dalalake_url: str) -> None:
        _string_key_secret = f'{api_key}:{secret}'
        _key_secret_bytes = _string_key_secret.encode("ascii")
        _encoded_key_secret_bytes = base64.b64encode(_key_secret_bytes)
        _encoded_key_secret = _encoded_key_secret_bytes.decode("ascii")

        self.datalake_interface = DatalakeInterface(_encoded_key_secret, dalalake_url)

    def upload_annotation_from_cocojson(self, file_path: str):
        datalake_logger.debug(f'file_name={file_path}')
        _annotation = Annotation(client=self)
        _annotation.upload_coco(file_path)

    def upload_modelrun_bbox_from_json(self, collection_name: str, model_id: str, file_path: str):
        datalake_logger.debug(f'file_name={file_path}')
        _model = Model(client=self)
        _model.upload_modelrun_json(collection_name, model_id, file_path, BOX_ANNOTATION)

    def upload_modelrun_polygon_from_json(self, collection_name: str, model_id: str, file_path: str):
        datalake_logger.debug(f'file_name={file_path}')
        _model = Model(client=self)
        _model.upload_modelrun_json(collection_name, model_id, file_path, POLYGON_ANNOTATION)

    def upload_modelrun_line_from_json(self, collection_name: str, model_id: str, file_path: str):
        datalake_logger.debug(f'file_name={file_path}')
        _model = Model(client=self)
        _model.upload_modelrun_json(collection_name, model_id, file_path, LINE_ANNOTATION)



# System Label Type
LABEL_CLASS = 1
LABEL_CLASS_WITH_ATTRIBUTES = 2

# Annotation Geometric Type
LINE_ANNOTATION = 'line'
POLYGON_ANNOTATION = 'polygon'
BOX_ANNOTATION = 'rectangle'

# Operation Data Meta Updates
OPERATION_TYPE_ANNOTATION = 1
OPERATION_MODE_HUMAN = 1
OPERATION_MODE_AUTO = 2

# Request Batch Sizes
META_UPDATE_REQUEST_BATCH_SIZE = 1000



import json

from datalake.keys import USERNAME, LABELS, META_UPDATES_ARRAY
import requests


class DatalakeInterface:

    def __init__(self, auth_token: str, dalalake_url: str):
        self.auth_token = auth_token
        self.dalalake_url = dalalake_url

    def create_datalake_label_coco(self, label, username='Python SDK'):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        payload = {
            USERNAME: username,
            LABELS: label,
        }
        url = f'{self.dalalake_url}/api/client/cocojson/import/label/create'

        try:
            response = requests.post(url=url, json=payload, headers=hed)
            return response.json()
        except requests.exceptions.RequestException as e:
            print("An exception occurred")
            print(e)

    def find_datalake_label_references(self, label_attribute_values_dict, username='Python SDK'):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        payload = {
            LABELS: label_attribute_values_dict,
            USERNAME: username
        }
        url = f'{self.dalalake_url}/api/client/system/label/references'

        try:
            response = requests.post(url=url, json=payload, headers=hed)
            return response.json()
        except requests.exceptions.RequestException as e:
            print("An exception occurred | find_datalake_label_references")
            print(e)

    def upload_metadata_updates(self, meta_updates, operation_type, operation_mode, operation_id):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        payload = {
            META_UPDATES_ARRAY: json.dumps(meta_updates),
        }
        url = f'{self.dalalake_url}/api/metadata/operationdata/{operation_type}/{operation_mode}/{operation_id}/update'

        try:
            response = requests.post(url=url, json=payload, headers=hed)
            return response.json()
        except requests.exceptions.RequestException as e:
            print("An exception occurred | find_datalake_label_references")
            print(e)
